DATA_TYPES = {
    'all': 'default',
    'iv': 'instantaneous Values',
    'uv': 'Unit Values',
    'rt': 'Real Time Values',
    'dv': 'Daily Values',
    'pk': 'Peak Measurements',
    'sv': 'Site Visits',
    'gw': 'Groundwater Levels',
    'qw': 'Water Quality Data',
    'id': 'Historical Instantaneous Values',
    'aw': 'Sites Monitored by the Active Groundwater Network',
    'ad': 'Sites included in Annual Water Data Reports',
}


SITE_TYPES = {
    'AG': 'Aggregate groundwater use',
    'AS': 'Aggregate surface-water-use',
    'AT': 'Atmosphere',
    'AW': 'Aggregate water-use establishment',
    'ES': 'Estuary',
    'FA': 'Facility',
    'FA-AWL': 'Animal waste lagoon',
    'FA-CI': 'Cistern',
    'FA-CS': 'Combined sewer',
    'FA-DV': 'Diversion',
    'FA-FON': 'Field, Pasture, Orchard, or Nursery',
    'FA-GC': 'Golf course',
    'FA-HP': 'Hydroelectric plant',
    'FA-LF': 'Landfill',
    'FA-OF': 'Outfall',
    'FA-PV': 'Pavement',
    'FA-QC': 'Laboratory or sample-preparation area',
    'FA-SEW': 'Wastewater sewer',
    'FA-SPS': 'Septic system',
    'FA-STS': 'Storm sewer',
    'FA-TEP': 'Thermoelectric plant',
    'FA-WDS': 'Water-distribution system',
    'FA-WIW': 'Waste injection well',
    'FA-WTP': 'Water-supply treatment plant',
    'FA-WU': 'Water-use establishment',
    'FA-WWD': 'Wastewater land application',
    'FA-WWTP': 'Wastewater-treatment plant',
    'GL': 'Glacier',
    'GW': 'Well',
    'GW-CR': 'Collector or Ranney type well',
    'GW-EX': 'Extensometer well',
    'GW-HZ': 'Hyporheic-zone well',
    'GW-IW': 'Interconnected wells',
    'GW-MW': 'Multiple wells',
    'GW-TH': 'Test hole not completed as a well',
    'LA': 'Land',
    'LA-EX': 'Excavation',
    'LA-OU': 'Outcrop',
    'LA-PLY': 'Playa',
    'LA-SH': 'Soil hole',
    'LA-SNK': 'Sinkhole',
    'LA-SR': 'Shore',
    'LA-VOL': 'Volcanic vent',
    'LK': 'Lake, Reservoir, Impoundment',
    'OC': 'Ocean',
    'OC-CO': 'Coastal',
    'SB': 'Subsurface',
    'SB-CV': 'Cave',
    'SB-GWD': 'Groundwater drain',
    'SB-TSM': 'Tunnel, shaft, or mine',
    'SB-UZ': 'Unsaturated zone',
    'SP': 'Spring',
    'SS': 'Specific Source',
    'ST': 'Stream',
    'ST-CA': 'Canal',
    'ST-DCH': 'Ditch',
    'ST-TS': 'Tidal stream',
    'WE': 'Wetland',
}
